package com.burtbeckwith.mvctest.interceptor;

import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.springframework.core.Ordered;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

/**
 * Simple interceptor that logs execution times and request parameters.
 *
 * @author Burt
 */
public class LoggingInterceptor extends HandlerInterceptorAdapter implements Ordered  {

	private static final String START_TIME_ATTRIBUTE =
		"LoggingInterceptor__START_TIME__";

	private static final String REQUEST_NUMBER_ATTRIBUTE =
		"LoggingInterceptor__REQUEST_NUMBER__";

	private static long _requestNumberCounter;

	private final Logger _log = Logger.getLogger(getClass());

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean preHandle(
			final HttpServletRequest request,
			final HttpServletResponse response,
			final Object handler) {

		if (!_log.isDebugEnabled()) {
			return true;
		}

		long start = System.currentTimeMillis();
		long currentRequestNumber = ++_requestNumberCounter;
		request.setAttribute(START_TIME_ATTRIBUTE, start);
		request.setAttribute(REQUEST_NUMBER_ATTRIBUTE, currentRequestNumber);

		String user = request.getRemoteUser();
		StringBuilder info = new StringBuilder();
		info.append("\n\tpreHandle request #").append(currentRequestNumber);
		info.append(": '");
		info.append(request.getServletPath());
		info.append("', user '").append(user == null ? "{N/A}" : user).append("'");
		info.append(", start ").append(new Date(start));
		info.append("\n\trequest data: ").append(extractNamesAndValues(request));
		_log.debug(info);

		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void postHandle(
			final HttpServletRequest request,
			final HttpServletResponse response,
			final Object handler,
			final ModelAndView modelAndView) {

		if (!_log.isDebugEnabled()) {
			return;
		}

		long start = (Long)request.getAttribute(START_TIME_ATTRIBUTE);
		long end = System.currentTimeMillis();
		long requestNumber = (Long)request.getAttribute(REQUEST_NUMBER_ATTRIBUTE);

		StringBuilder info = new StringBuilder();
		info.append("\n\tpostHandle request #").append(requestNumber);
		info.append(" end ").append(new Date(end));
		info.append(", total time ").append(end - start);
		info.append("ms, handler: ").append(handler.getClass().getSimpleName());
		_log.debug(info);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void afterCompletion(
			final HttpServletRequest request,
			final HttpServletResponse response,
			final Object handler,
			final Exception e) {

		if (!_log.isDebugEnabled()) {
			return;
		}

		long start = (Long)request.getAttribute(START_TIME_ATTRIBUTE);
		long end = System.currentTimeMillis();
		long requestNumber = (Long)request.getAttribute(REQUEST_NUMBER_ATTRIBUTE);

		StringBuilder info = new StringBuilder();
		info.append("\n\tafterCompletion request #").append(requestNumber);
		info.append(" end ").append(new Date(end));
		info.append(", total time ").append(end - start).append("ms");

		if (e == null) {
			_log.debug(info);
		}
		else {
			_log.debug(info + "\n\texception: " + e.getMessage(), e);
		}
	}

	/**
	 * {@inheritDoc}
	 * @see org.springframework.core.Ordered#getOrder()
	 */
	public int getOrder() {
		return Integer.MAX_VALUE;
	}
	
	/**
	 * Extract request parameters into name/value pairs. Different from
	 * <code>HttpServletRequest.getParameterMap()</code> since that method returns
	 * arrays and this returns single values.
	 * @param request  the request
	 * @return  keys are parameter names, values are request parameter values
	 */
	private Map<String, String> extractNamesAndValues(final HttpServletRequest request) {
		Map<String, String> namesAndValues = new HashMap<String, String>();
		Enumeration<?> enumNames = request.getParameterNames();
		while (enumNames.hasMoreElements()) {
			String name = (String)enumNames.nextElement();
			namesAndValues.put(name, request.getParameter(name));
		}
		return namesAndValues;
	}
}
